/* Copyright 2004,2005,2006 Thomas "thisnukes4u" Coppi
   If you use this program, please send me an e-mail at thisnukes4u@gmail.com
   You may do whatever you wish with this program except for selling it, claiming it as your own,
   or modifying it without releasing the source code (the listed restrictions are not mutally exclusive,
   they all apply). If you do not agree with the above terms, you may optionally distribute and use 
   this software under the GNU GPL, the terms of which are included in the COPYING file. */


#include <gtk/gtk.h>
#include <gdk/gdk.h>

#include <stdio.h>
#include <stdlib.h>

#include "config.h"
#include "eggtrayicon.h"
#include "docklet.h"

/* please note that I borrowed much of this off of the gaim-docklet plugin, so 
   I wish to thank them very much for that */

static void docklet_x11_embedded_cb(GtkWidget *widget, void *data);
static void docklet_x11_destroyed_cb(GtkWidget *widget, void *data);
static void docklet_x11_update_icon(enum docklet_status icon);

static enum docklet_status status = normal;

extern GtkWidget *window; /* from gui.c */
extern char *title; /* from gui.c */
extern GtkTooltips *tooltip; /* from gui.c */

static EggTrayIcon *docklet = NULL;
static GtkWidget *image = NULL;

void docklet_embedded()
{
	docklet_x11_update_icon(status);
}
	
	
static void docklet_x11_update_icon(enum docklet_status icon)
{	
	gtk_image_set_from_file(GTK_IMAGE(image),"/usr/share/gfeed/icon.png");
}	

/*displays a simple about dialog */
static void about_clicked(GtkWidget *widget, gpointer data)
{
	GtkWidget *dialog;
	gchar *about =  g_malloc0(400);

	snprintf(about,(size_t)400,
		"gfeed version %s  \nCopyright 2004,2005,2006 Thomas \"thisnukes4u\" Coppi  \n\n\"Any society that would give up a little liberty to gain a little security will deserve neither and lose both.\" \n\t- Benjamin Franklin",
		PACKAGE_VERSION);
	
	dialog = gtk_message_dialog_new(GTK_WINDOW(window),
				   GTK_DIALOG_DESTROY_WITH_PARENT,
				   GTK_MESSAGE_INFO,
				   GTK_BUTTONS_CLOSE,
				   about);
			gtk_window_set_title(GTK_WINDOW(dialog),"About gfeed");
	   		gtk_dialog_run(GTK_DIALOG(dialog));
  			gtk_widget_destroy(dialog);

	free(about);
}

static void docklet_clicked(GdkEventButton *event)
{
	GtkWidget *menu;
	GtkWidget *quit,*about;

	if(event->type != GDK_BUTTON_PRESS) return;
	else {
		if(event->button == 1) { /* hide the main window if it is shown, else show it */
			if(GTK_WIDGET_VISIBLE(window)) gtk_widget_hide(window);
			else gtk_widget_show(window); 
		}
		else if(event->button == 3) { /* show a menu containing about and quit*/
			menu = gtk_menu_new();
			about = gtk_image_menu_item_new_with_label("About");
			quit = gtk_image_menu_item_new_with_label("Quit");
			g_signal_connect(G_OBJECT(quit),"activate",G_CALLBACK(gtk_main_quit),NULL);
			g_signal_connect(G_OBJECT(about),"activate",G_CALLBACK(about_clicked),NULL);
			gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(quit),
 					gtk_image_new_from_stock(GTK_STOCK_QUIT,GTK_ICON_SIZE_MENU));
			gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(about),
#if GTK_CHECK_VERSION(2,6,0)		
					gtk_image_new_from_stock(GTK_STOCK_ABOUT,GTK_ICON_SIZE_MENU));	
#else			
					gtk_image_new_from_stock(GTK_STOCK_HELP,GTK_ICON_SIZE_MENU));
#endif
			gtk_menu_shell_insert(GTK_MENU_SHELL(menu),about,0);
			gtk_menu_shell_insert(GTK_MENU_SHELL(menu),quit,1);
			gtk_widget_show_all(menu);
			gtk_menu_popup(GTK_MENU(menu),NULL,NULL,NULL,NULL,event->button,gtk_get_current_event_time());
		}	

	}

}

static void docklet_x11_clicked_cb(GtkWidget *button, GdkEventButton *event, void *data)
{
	docklet_clicked(event);
}

void docklet_x11_create()
{
        GtkWidget *box;
        
	docklet = egg_tray_icon_new("gfeed");
        box = gtk_event_box_new();
        image = gtk_image_new();

        g_signal_connect(G_OBJECT(docklet), "embedded", G_CALLBACK(docklet_x11_embedded_cb), NULL);
        g_signal_connect(G_OBJECT(docklet), "destroy", G_CALLBACK(docklet_x11_destroyed_cb), NULL);
	g_signal_connect(G_OBJECT(box), "button-press-event", G_CALLBACK(docklet_x11_clicked_cb), NULL);

        gtk_container_add(GTK_CONTAINER(box), image);
        gtk_container_add(GTK_CONTAINER(docklet), box);

        if(!gtk_check_version(2,4,0))
                g_object_set(G_OBJECT(box), "visible-window", FALSE, NULL);

	gtk_tooltips_set_tip(tooltip,GTK_WIDGET(docklet),title,NULL);
	
        gtk_widget_show_all(GTK_WIDGET(docklet));

        /* ref the docklet before we bandy it about the place */
        g_object_ref(G_OBJECT(docklet));
}

static void docklet_x11_destroyed_cb(GtkWidget *widget, void *data)
{
        g_object_unref(G_OBJECT(docklet));
        docklet = NULL;
	
	/* recreate ourselves, since the most likely reason we were killed
	 * was because the systray application was restarted/killed. */
	
	docklet_x11_create();
}

static void docklet_x11_embedded_cb(GtkWidget *widget, void *data)
{
        docklet_embedded();
}

void docklet_update_title(char *title)
{
	gtk_tooltips_set_tip(tooltip,GTK_WIDGET(docklet),title,NULL);
}

