/* Copyright 2004,2005,2006 Thomas "thisnukes4u" Coppi
   If you use this program, please send me an e-mail at thisnukes4u@gmail.com
   You may do whatever you wish with this program except for selling it, claiming it as your own,
   or modifying it without releasing the source code (the listed restrictions are not mutally exclusive,
   they all apply). If you do not agree with the above terms, you may optionally distribute and use 
   this software under the GNU GPL, the terms of which are included in the COPYING file. */

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkx.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>

#include "rssfeed.h"
#include "conf.h"
#include "gui.h"
#include "eggtrayicon.h"
#include "docklet.h"
#include "error.h"
#include "log.h"
#include "config.h"

#define OPAQUE	0xffffffff
#define OPACITY	"_NET_WM_WINDOW_OPACITY"

/* FIXME If you are reading this, before you continue, you should know that this code 
   is a mess(getting better as of 6-12-05), and it is quite horrid to have to add things to(getting better).  So, if you
   plan to do anything to it, please clean it up first and send the patch in FIXME */

/* Widget Variables */
int size_x;
int size_y;
GtkWidget *window;
char *title;
GtkTooltips *tooltip;
static GtkWidget *box1;
static GtkWidget *box2;
static GtkWidget *scrolledwindow;
static GtkTextBuffer *buffer;
static GtkWidget *view;
static GtkWidget *update;
static GtkWidget *quit;
static GtkTextTag *link_tag[MAX_ITEMS];
static GtkTextTag *title_tag;
static GtkTextIter end;
static GtkWidget *copy_menu;
static GtkWidget *copy_main;
static GtkWidget *open_main;
PangoFontDescription *font_desc;

extern FILE *logfp;
extern unsigned int logging;

/* Local Function Prototypes */
static void set_buffer(RssFeed *feed);
static void update_feed_and_refresh_buffer(GtkWidget *widget, RssFeed *data);
static void link_event(GtkTextTag *texttag,GObject *arg1, GdkEvent *event, GtkTextIter *arg2, gpointer user_data);
static void copy_selected(GtkMenuItem *item, gpointer user_data);
static gboolean update_timeout(gpointer *feed);
//static gboolean update_first(gpointer *feed);
static void open_selected(GtkMenuItem *item, gpointer user_data);
static void onquit(gpointer data);	

/* creates and displays the main window */
void open_main_window(RssFeed *feed, double opacity, Config *config)
{ 
	int i;
	char gfeed[100] = "gfeed::";
	char temp[MAX_ITEMS+6];
	unsigned int opac;
	char *font_and_size = NULL;
	GdkPixbuf *icon;

	size_x = config->geom_x;
	size_y = config->geom_y;	
	
	/* Make sure that the title is
	   a real one */
	if(!strcmp(config->title,"a"))
		title = strncat(gfeed,feed->channel_title,93);
	else
		title = strncat(gfeed,config->title,93);
	
	/* setup tooltips */
	tooltip = gtk_tooltips_new();
	gtk_tooltips_enable(tooltip);
	
	/* create the system tray icon */
	docklet_x11_create();
	
	/* sets up the initial widgets */
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	icon = gdk_pixbuf_new_from_file("/usr/share/gfeed/icon.png",NULL);
	gtk_window_set_icon(GTK_WINDOW(window),icon);
	gtk_window_set_title(GTK_WINDOW(window),title);
	gtk_window_set_default_size(GTK_WINDOW(window),config->geom_x,config->geom_y);
	gtk_window_set_decorated(GTK_WINDOW(window),config->decorated);
	scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),GTK_POLICY_NEVER,GTK_POLICY_AUTOMATIC);
	view = gtk_text_view_new();
	update = gtk_button_new_with_label("Update");
	if(!config->noquit) {
		quit = gtk_button_new_with_label("Quit");
		gtk_tooltips_set_tip(tooltip,quit,"Quit",NULL);
	}
	
	gtk_tooltips_set_tip(tooltip,update,"Update the feed now",NULL);
	gtk_tooltips_enable(tooltip);

	/* signals and a little packing */
	gtk_quit_add(0,(GtkFunction )onquit,feed);
	g_signal_connect(G_OBJECT(window), "delete_event",G_CALLBACK(gtk_main_quit), NULL);
	g_signal_connect(G_OBJECT(update),"pressed",G_CALLBACK(update_feed_and_refresh_buffer),feed);
	if(!config->noquit) g_signal_connect(G_OBJECT(quit),"pressed",G_CALLBACK(gtk_main_quit),NULL);
	box1 = gtk_vbox_new(FALSE,0);
	box2 = gtk_hbox_new(FALSE,0);
	gtk_container_add(GTK_CONTAINER(window), box1);

	/* Setup the buffer */
	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
	gtk_text_buffer_get_iter_at_offset(buffer, &end, 0);

	/* Create all of the tags for links and such */
	for(i=0;i<MAX_ITEMS;i++) {
		sprintf(temp,"link%d",i);
		link_tag[i] = gtk_text_buffer_create_tag(buffer,temp,
					     "underline",PANGO_UNDERLINE_SINGLE,
					     "foreground",config->lcolor,
					     "size",(config->font_size_i) * PANGO_SCALE,
					     NULL);
	}
	title_tag = gtk_text_buffer_create_tag(buffer,"title",
			     "weight", PANGO_WEIGHT_BOLD,
			     "foreground",config->sfgcolor,
			     "size",(config->font_size_i + 2) * PANGO_SCALE,
			     NULL);

	free(config->sfgcolor);
	
	/* Setup the buffer */
	set_buffer(feed);
	gtk_text_view_set_editable(GTK_TEXT_VIEW(view), FALSE);
	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(view),GTK_WRAP_WORD);	
	gtk_widget_modify_base(view,GTK_STATE_NORMAL,&config->bgcolor);	
	
	/* Font stuff */
	font_and_size = g_malloc0(1000);
	strncat(font_and_size,config->font_face,900);
	strncat(font_and_size,config->font_size,900);
	font_desc = pango_font_description_from_string(font_and_size);
	gtk_widget_modify_font(view,font_desc);
	free(font_and_size);
	font_and_size = NULL;
	
	/* pack our containers with widgets */
	gtk_container_add(GTK_CONTAINER(scrolledwindow), view);  
	gtk_box_pack_start(GTK_BOX(box1),scrolledwindow, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(box1),box2,FALSE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(box2),update,TRUE,TRUE,0);
	if(!config->noquit) gtk_box_pack_start(GTK_BOX(box2),quit,TRUE,TRUE,0); 

	/* Show the window and widgets */
	gtk_widget_show_all(window);

	update_feed_and_refresh_buffer(NULL,feed);
	if(config->log) {
		logfp = open_logfile(feed);
		write_log_item(logfp,"INFO:Opening feed");
		logging = 1;
	}	
	
	/* TRANSPARANCY 
	I got most of this from the transset program on freedesktop.org, just added
	in the GDK stuff */
	opac = (unsigned int)(opacity * OPAQUE);
	
	XChangeProperty(GDK_DRAWABLE_XDISPLAY(GTK_WIDGET(window)->window),
		  GDK_WINDOW_XWINDOW(GTK_WIDGET(window)->window),
		  XInternAtom(GDK_DRAWABLE_XDISPLAY(GTK_WIDGET(window)->window), OPACITY, False),
		  XA_CARDINAL, 32, PropModeReplace,(unsigned char *)&opac,1L);
	XSync(GDK_DRAWABLE_XDISPLAY(GTK_WIDGET(window)->window), False);

	gtk_window_resize(GTK_WINDOW(window),config->geom_x,config->geom_y);
	
	/* Reshow the window */
	gtk_widget_show_all(window);

	/* timeout to update the feed */
	g_timeout_add(config->update*60000,(GSourceFunc)update_timeout,feed);

	/* timeout to update the feed for the first time */
	//g_timeout_add(1,(GSourceFunc)update_first,feed);
	
	/* main gtk loop */
	gtk_main();     	
}

/* sets the text buffer to the proper text and formating */
static void set_buffer(RssFeed *feed)
{		
	/* just some variables we are going to need */
	int i,tmp;
	int currnum = feed->currnum;
	GtkTextIter iter;
	char *title = NULL;
	char *link = NULL; 
	char *description = NULL; 
	char *temp = NULL;

	if(feed->channel_title)
		title = g_strdup(feed->channel_title);
	else
		title = g_strdup("No Title");
	
	if(feed->channel_link)	
		link = g_strdup(feed->channel_link);
	else
		link = g_strdup("No Link");
	if(feed->channel_desc)
		description = g_strdup(feed->channel_desc);
	else
		description = g_strdup(feed->channel_link);
	
	/* Make sure the newly allocated memory is not NULL */
	if(!title) gfeed_error(logfp, MEM_ALLOC);
	if(!link) gfeed_error(logfp, MEM_ALLOC);
	if(!description) gfeed_error(logfp, MEM_ALLOC);	

	/* resize the memory to give us one more byte of space */
	tmp = strlen(title);
	title = realloc(title,tmp+2);
	tmp = strlen(link);
	link = realloc(link,tmp+2);
	tmp = strlen(description);
	description = realloc(description,tmp+2);

	/* now we can add that extra \n */	
	title = strcat(title,"\n");
	link = strcat(link,"\n");
	description = strcat(description,"\n");

	/* get start of buffer; each insertion will revalidate the
	iterator to point to just after the inserted text.
	*/
	gtk_text_buffer_get_iter_at_offset(buffer, &iter, 0);

	/* delete the old buffer */
	gtk_text_buffer_delete(buffer,&iter,&end);
  
	/* Title */
	gtk_text_buffer_insert_with_tags(buffer,&iter,title,-1,title_tag,NULL);

	/* Link */
	g_signal_connect(G_OBJECT(title_tag),"event",G_CALLBACK(link_event),title_tag);
	g_object_set_data(G_OBJECT(title_tag),"link",g_strdup(link));
	g_object_set_data(G_OBJECT(title_tag),"description",g_strdup(description));
	
	free(title);
	free(description);
	free(link);	
	title = description = link = NULL;  
	
	/* For each item, we output a set of strings */
	for(i=0;i<currnum;i++) {
		/* Output title, attach description and link values to the link_tag and connect signal for 
		   clicking on the text */
		if(!feed->item_title[i]) 
			temp = g_strdup("ERROR:NO TITLE SPECIFIED");
		else temp = g_strdup(feed->item_title[i]);
		
		if(!temp) 
			gfeed_error(logfp, MEM_ALLOC);
		
		title = g_malloc0(502);
		
		if(!title) 
			gfeed_error(logfp, MEM_ALLOC);
		
		title = strcpy(title,"--");
		title = strncat(title,temp,490);
		free(temp);
		title = strcat(title,"\n");
		temp = g_strdup(feed->item_link[i]);
		
		if(!temp) 
			gfeed_error(logfp, MEM_ALLOC);
		
		temp = (char *)realloc(temp,1000);
		temp = strcat(temp,"\n");
		gtk_text_buffer_insert_with_tags(buffer,&iter,title,-1,link_tag[i],NULL);
		g_signal_connect(G_OBJECT(link_tag[i]),"event",G_CALLBACK(link_event),link_tag[i]);
		g_object_set_data(G_OBJECT(link_tag[i]),"link",g_strdup(temp));
		
		if(!feed->item_desc[i] || feed->item_desc[i] == "") {
			g_object_set_data(G_OBJECT(link_tag[i]),"description",
				g_strdup("ERROR: NO DESCRIPTION SPECIFIED"));
		}
		else g_object_set_data(G_OBJECT(link_tag[i]),"description",g_strdup(feed->item_desc[i]));
		
		if(!feed->item_date[i] || feed->item_date[i] == "") {
			g_object_set_data(G_OBJECT(link_tag[i]),"date",
				g_strdup("ERROR: NO DATE SPECIFIED"));
		}
		else g_object_set_data(G_OBJECT(link_tag[i]),"date",g_strdup(feed->item_date[i]));
		
		free(temp);
		free(title);
		temp = title = NULL;
	}
	end = iter;
}

/* Updates the feed by freeing all its data and reparsing it
   Refreshes buffer by calling set_buffer() with the new feed */
static void update_feed_and_refresh_buffer(GtkWidget *widget, RssFeed *data)
{
	int i;
	char gfeed[100] = "gfeed::";

	static unsigned short int pressed = 0;
	
	if(pressed == 0) {
		pressed = 1;
	
		write_log_item(logfp,"INFO:Updating feed...");
		
		/* Free unused data from the last feed */
		free_rssfeed(data);
			
		if(g_object_get_data(G_OBJECT(title_tag),"description"))
			free(g_object_get_data(G_OBJECT(title_tag),"description"));

		if(g_object_get_data(G_OBJECT(title_tag),"link"))
			free(g_object_get_data(G_OBJECT(title_tag),"link"));
		
		g_signal_handlers_disconnect_by_func(G_OBJECT(title_tag),G_CALLBACK(link_event),title_tag);
		
		for(i=0;i<data->currnum;i++) {
			g_signal_handlers_disconnect_by_func(G_OBJECT(link_tag[i]),G_CALLBACK(link_event),link_tag[i]); 
			free(g_object_get_data(G_OBJECT(link_tag[i]),"description"));
			free(g_object_get_data(G_OBJECT(link_tag[i]),"link"));
			free(g_object_get_data(G_OBJECT(link_tag[i]),"date"));
		}

		/* Parse the feed again, and update
		   the buffer */
		if(parse(data) == -1) gfeed_error(logfp, RES_UNAV);
		set_buffer(data);
		
		title = strncat(gfeed,data->channel_title,93);
		gtk_window_set_title(GTK_WINDOW(window),title);

		docklet_update_title(title);
		
		gtk_widget_show_all(window);

		pressed = 0;
	}
}

/* Every X minutes, this is called to update
   the buffer */
static gboolean update_timeout(gpointer *feed)
{
	update_feed_and_refresh_buffer(NULL,(RssFeed *)feed);
	return TRUE;
}

/*
static gboolean update_first(gpointer *feed)
{	
	update_feed_and_refresh_buffer(NULL,(RssFeed *)feed);
	return FALSE;
}
*/

/* callback */
static void link_event(GtkTextTag *texttag,GObject *arg1, GdkEvent *event, GtkTextIter *arg2, gpointer user_data)
{
	gchar *url;
	GtkWidget *dialog;
	
	/* If button 1(left-click) is pressed, popup a dialog that displays the description.  If button
	 3(right-click) is pressed, popup a menu to copy the link */
	GdkEventButton *event_button = (GdkEventButton *) event;
	if(event->type == GDK_BUTTON_RELEASE) {
		if(event_button->button == 1) {
			url = g_strdup((gchar *)g_object_get_data(G_OBJECT(user_data),"description"));
			dialog = gtk_message_dialog_new(GTK_WINDOW(window),
				   GTK_DIALOG_DESTROY_WITH_PARENT,
				   GTK_MESSAGE_INFO,
				   GTK_BUTTONS_CLOSE,
				   url);
			
			gtk_window_set_title(GTK_WINDOW(dialog),g_object_get_data(G_OBJECT(user_data),"date"));
	   		gtk_dialog_run(GTK_DIALOG(dialog));
  			gtk_widget_destroy(dialog);
			
			free(url);
			url = NULL;	
		}
		else if(event_button->button == 3) {
			copy_menu = gtk_menu_new();
			
			copy_main = gtk_menu_item_new();
			open_main = gtk_menu_item_new();

			url = g_strdup((gchar *)g_object_get_data(G_OBJECT(user_data),"link"));
			copy_main = gtk_image_menu_item_new_with_label("Copy Link Location");
			
			gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(copy_main),
						    gtk_image_new_from_stock(GTK_STOCK_COPY,GTK_ICON_SIZE_MENU));
		
			gtk_menu_shell_insert(GTK_MENU_SHELL(copy_menu),copy_main,0);
			g_signal_connect(G_OBJECT(copy_main),"activate",G_CALLBACK(copy_selected),
							g_strdup(url));
			
			open_main = gtk_image_menu_item_new_with_label("Open in New Firefox Tab");
			gtk_menu_shell_insert(GTK_MENU_SHELL(copy_menu),open_main,0);
			g_signal_connect(G_OBJECT(open_main),"activate",G_CALLBACK(open_selected),
							g_strdup(url));
			
			gtk_widget_show_all(copy_menu);
			gtk_menu_popup(GTK_MENU(copy_menu),NULL,NULL,NULL,NULL,event_button->button,0);
			
			free(url);
			url = NULL;	
		}
	}
}

/* callback */
static void copy_selected(GtkMenuItem *item, gpointer user_data)
{
	/* just copies the link selected to the clipboard(CLIPBOARD and PRIMARY) */
	GtkClipboard *board = gtk_clipboard_get_for_display(gdk_display_get_default(),GDK_SELECTION_CLIPBOARD);
	gtk_clipboard_set_text(board,(char *)user_data,-1);
	board = gtk_clipboard_get_for_display(gdk_display_get_default(),GDK_SELECTION_PRIMARY);
	gtk_clipboard_set_text(board,(char *)user_data,-1);
	
	free(user_data);
	user_data = NULL;
}

/* callback */
static void open_selected(GtkMenuItem *item, gpointer user_data)
{
	char *command;

	command = calloc(300,1);
	
	snprintf(command,300,"firefox -remote \"openurl(%s,new-tab)\"",(char *)user_data);
	
	system(command);
	free(command);
	free(user_data);
	user_data = NULL;
}

/* called on gtk_main_quit */
static void onquit(gpointer data)
{
	free_rssfeed(data);
	fclose(logfp);
}
