/* Copyright 2004,2005,2006 Thomas "thisnukes4u" Coppi
   If you use this program, please send me an e-mail at thisnukes4u@gmail.com
   You may do whatever you wish with this program except for selling it, claiming it as your own,
   or modifying it without releasing the source code (the listed restrictions are not mutally exclusive,
   they all apply). If you do not agree with the above terms, you may optionally distribute and use 
   this software under the GNU GPL, the terms of which are included in the COPYING file */

#include <gtk/gtk.h>

#include "config.h"
#include "rssfeed.h"
#include "conf.h"
#include "gui.h"
#include "error.h"
#include "log.h"

#include <stdio.h>
#include <unistd.h>
#include <signal.h>

FILE *logfp;
unsigned int logging = 0; /* If this is 1, that means that we are logging to a dotfile, not just to stderr */

void segv(int sig);

int main(int argc, char **argv) 
{ 
	RssFeed feed;
	Config *conf = NULL;
	char *location = NULL;
	int i, show_titles = 0, show_desc = 0, show_links = 0;
	float trans = 0;
	  
	/* the standard gtk init */
	gtk_init(&argc,&argv);
	
	/* set up our signal handler */
	signal(SIGSEGV,segv);
	
	logfp = stderr;
	
	if(argc <= 1) {
		printf("Usage: %s [feed config location] \n-o transparency %% \n-f feed location(no feed config necessary, defaults will be used)\n-v Version\n-t Output feed titles in a parsable manner(to stdout)\n-d Output feed item descriptions in a parsable manner(only meaningful when combined with -t)\n-l Output feed link(only meaningful when combined with -t)\n", argv[0]);
	   	return -1;
	}
	else location = g_strdup(argv[1]);

	/* parse the options */
	for(i=0;i<argc;i++) {
		switch(getopt(argc,argv,"o: f: v t d l")) {
			case 'o': trans = atof(optarg); break;
			case 'f': conf = read_default_config(optarg); break;
			case 't': show_titles = 1; break;
			case 'd': if(show_titles) show_desc = 1; break;
			case 'l': if(show_titles) show_links = 1; break;
			case 'v': printf("gfeed RSS/RDF feed reader version %s\nLiscense terms:\nYou may do whatever you wish with this program except for selling it, claiming it as your own,or modifying it without releasing the source code (the listed restrictions are not mutally exclusive,they all apply). If you do not agree with the above terms, you may optionally distribute and use this software under the GNU GPL, the terms of which are included in the COPYING file\n",PACKAGE_VERSION); break;
		}
	 }
		  
	  
	/*parse the config file if need be*/
	if(!conf) conf = read_config(location);
       	
	/* a "null" feed location means that no location was specified.  Otherwise, we can either output to the console
   		or start up the gui
     	*/		
	if(strcmp(conf->feed_loc,"null")) { 
		if(show_titles) {
			new_rssfeed(conf->feed_loc, &feed);
			if(parse(&feed) == -1) gfeed_error(logfp,RES_UNAV);
			for(i=0;i<feed.currnum;i++) {
				printf("Title-%i:%s\n",i,feed.item_title[i]);
				if(show_desc) printf("Description-%i:%s\n",i,feed.item_desc[i]);
				if(show_links) printf("Link-%i:%s\n",i,feed.item_link[i]);
			}	
		}
		else { 
			/* parse the feed and display the window */
			new_rssfeed(conf->feed_loc, &feed);
			feed.channel_link = g_strdup("");
			feed.channel_desc = g_strdup("");
			
			feed.channel_title = g_strdup("Please wait while gfeed is loading...");
			
			open_main_window(&feed,((trans == 0) ? trans : (double)100), conf);
		}
		free(location);
		return 0;
	 }
	  else
		printf("No feed location has been specified, bailing.\n");  
	  	return -1;
}

void segv(int sig)
{
	printf("gfeed has crashed.  Please attempt to replicate the problem and report it on the gfeed sourceforge site (http://sourceforge.net/projects/gfeed) or email me at thisnukes4u@gmail.com  A backtrace using gdb would also be appreciated if you know how to make one.\n");

	fclose(logfp);
	exit(-1);
}

